﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using TSLab.Script.Helpers;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable CompareOfFloatsByEqualityOperator
namespace TSLab.Script.Handlers
{
    public static class ADXHelper
    {
        public static IList<double> CalcDIP(ISecurity source, int period)
        {
            int count = source.Bars.Count;
            IList<double> diP = new double[count];

            var high = source.HighPrices;
            var low = source.LowPrices;

            var atr = Series.EMA(Series.TrueRange(source.Bars), period);
            Debug.Assert(atr != null, "atr != null");

            for (int i = 1; i < count; i++)
            {
                var dmP = high[i] - high[i - 1];
                var dmM = low[i - 1] - low[i];
                if ((dmP < 0 && dmM < 0) || dmP == dmM)
                {
                    dmP = dmM = 0;
                }
                if (dmM > dmP)
                {
                    dmP = 0;
                }
                diP[i] = dmP;
            }
            diP = Series.EMA(diP, period);
            for (int i = 1; i < count; i++)
            {
                diP[i] = atr[i] == 0 ? 0 : diP[i] / atr[i];
            }
            return diP;
        }

        public static IList<double> CalcDIM(ISecurity source, int period)
        {
            int count = source.Bars.Count;
            IList<double> diM = new double[count];

            var high = source.HighPrices;
            var low = source.LowPrices;

            var atr = Series.EMA(Series.TrueRange(source.Bars), period);

            for (int i = 1; i < count; i++)
            {
                var dmP = high[i] - high[i - 1];
                var dmM = low[i - 1] - low[i];
                if ((dmP < 0 && dmM < 0) || dmP == dmM)
                {
                    dmP = dmM = 0;
                }
                if (dmP > dmM)
                {
                    dmM = 0;
                }
                diM[i] = dmM;
            }
            diM = Series.EMA(diM, period);
            for (int i = 1; i < count; i++)
            {
                diM[i] = atr[i] == 0 ? 0 : diM[i] / atr[i];
            }
            return diM;
        }

        public static IList<double> CalcADX(IList<double> source1, IList<double> source2, int period)
        {
            int count = source1.Count;
            IList<double> dx = new double[count];

            for (int i = 1; i < count; i++)
            {
                dx[i] = source1[i] == 0 && source2[i] == 0
                             ? 0
                             : Math.Abs(source1[i] - source2[i]) / (source1[i] + source2[i]) * 100;
            }
            dx = Series.EMA(dx, period);
            return dx;
        }
    }

    //[HandlerName("+DI")]
    [HandlerCategory(HandlerCategories.Indicators)]
    public sealed class DIP : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            return ADXHelper.CalcDIP(source, Period);
        }

        public IContext Context { get; set; }
    }

    //[HandlerName("-DI")]
    [HandlerCategory(HandlerCategories.Indicators)]
    public sealed class DIM : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            return ADXHelper.CalcDIM(source, Period);
        }

        public IContext Context { get; set; }
    }

    //[HandlerName("ADX")]
    [HandlerCategory(HandlerCategories.Indicators)]
    public sealed class ADXFull : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            var dip = ADXHelper.CalcDIP(source, Period);
            var dim = ADXHelper.CalcDIM(source, Period);
            return ADXHelper.CalcADX(dip, dim, Period);
        }

        public IContext Context { get; set; }
    }

    //[HandlerName("ADX (Old)")]
    [HandlerCategory(HandlerCategories.Indicators)]
    public class ADX : BasePeriodIndicatorHandler, IDoubleAccumHandler
    {
        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return ADXHelper.CalcADX(source1, source2, Period);
        }
    }
}
