﻿using System;
using System.Collections.Generic;
using TSLab.Script.Helpers;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable ClassNeverInstantiated.Global
namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.TradeMath)]
    [InputsCount(2, 6)]
    public sealed class Max : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return Math.Max(source1, source2);
        }
        public double Execute(double source1, double source2, double source3)
        {
            return Execute(Math.Max(source1, source2), source3);
        }

        public double Execute(double source1, double source2, double source3, double source4)
        {
            return Execute(Math.Max(source1, source2), source3, source4);
        }

        public double Execute(double source1, double source2, double source3, double source4, double source5)
        {
            return Execute(Math.Max(source1, source2), source3, source4, source5);
        }

        public double Execute(double source1, double source2, double source3, double source4, double source5, double source6)
        {
            return Execute(Math.Max(source1, source2), source3, source4, source5, source6);
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Max(source1, source2);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3)
        {
            return Series.Max(Execute(source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4)
        {
            return Series.Max(Execute(source4, source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4, IList<double> source5)
        {
            return Series.Max(Execute(source5, source4, source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4, IList<double> source5, IList<double> source6)
        {
            return Series.Max(Execute(source6, source5, source4, source3, source2), source1);
        }

        #endregion
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    [InputsCount(2, 6)]
    public sealed class Min : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return Math.Min(source1, source2);
        }
        public double Execute(double source1, double source2, double source3)
        {
            return Execute(Math.Min(source1, source2), source3);
        }

        public double Execute(double source1, double source2, double source3, double source4)
        {
            return Execute(Math.Min(source1, source2), source3, source4);
        }

        public double Execute(double source1, double source2, double source3, double source4, double source5)
        {
            return Execute(Math.Min(source1, source2), source3, source4, source5);
        }

        public double Execute(double source1, double source2, double source3, double source4, double source5, double source6)
        {
            return Execute(Math.Min(source1, source2), source3, source4, source5, source6);
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Min(source1, source2);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3)
        {
            return Series.Min(Execute(source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4)
        {
            return Series.Min(Execute(source4, source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4, IList<double> source5)
        {
            return Series.Min(Execute(source5, source4, source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4, IList<double> source5, IList<double> source6)
        {
            return Series.Min(Execute(source6, source5, source4, source3, source2), source1);
        }

        #endregion
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    [InputsCount(2, 6)]
    public sealed class Add : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return source1 + source2;
        }

        public double Execute(double source1, double source2, double source3)
        {
            return source1 + source2 + source3;
        }

        public double Execute(double source1, double source2, double source3, double source4)
        {
            return source1 + source2 + source3 + source4;
        }

        public double Execute(double source1, double source2, double source3, double source4, double source5)
        {
            return source1 + source2 + source3 + source4 + source5;
        }

        public double Execute(double source1, double source2, double source3, double source4, double source5, double source6)
        {
            return source1 + source2 + source3 + source4 + source5 + source6;
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Add(source1, source2);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3)
        {
            return Series.Add(Execute(source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4)
        {
            return Series.Add(Execute(source4, source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4, IList<double> source5)
        {
            return Series.Add(Execute(source5, source4, source3, source2), source1);
        }

        public IList<double> Execute(IList<double> source1, IList<double> source2, IList<double> source3, IList<double> source4, IList<double> source5, IList<double> source6)
        {
            return Series.Add(Execute(source6, source5, source4, source3, source2), source1);
        }

        #endregion
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class Sub : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return source1 - source2;
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Sub(source1, source2);
        }

        #endregion
    }

    [HandlerCategory(HandlerCategories.TradeMath, "LogicalNotTemplate", false)]
    public sealed class Not : IBoolConvertor
    {
        #region IBoolConvertor Members

        public IList<bool> Execute(IList<bool> src)
        {
            var dst = new bool[src.Count];
            for (int i = 0; i < src.Count; i++)
            {
                dst[i] = !src[i];
            }
            return dst;
        }

        public bool Execute(bool source, int num)
        {
            return !source;
        }

        #endregion
    }

    [HandlerCategory(HandlerCategories.TradeMath, "LogicalAndTemplate", false)]
    [InputsCount(2, 6)]
    public sealed class And : IBoolAccum
    {
        #region IBoolAccum Members

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2)
        {
            return Series.And(src1, src2);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3)
        {
            return Execute(Series.And(src1, src2), src3);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3, IList<bool> src4)
        {
            return Execute(Series.And(src1, src2), src3, src4);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3, IList<bool> src4, IList<bool> src5)
        {
            return Execute(Series.And(src1, src2), src3, src4, src5);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3, IList<bool> src4, IList<bool> src5, IList<bool> src6)
        {
            return Execute(Series.And(src1, src2), src3, src4, src5, src6);
        }

        public bool Execute(bool source1, bool source2)
        {
            return source1 && source2;
        }

        public bool Execute(bool source1, bool source2, bool source3)
        {
            return source1 && source2 && source3;
        }

        public bool Execute(bool source1, bool source2, bool source3, bool source4)
        {
            return source1 && source2 && source3 && source4;
        }

        public bool Execute(bool source1, bool source2, bool source3, bool source4, bool source5)
        {
            return source1 && source2 && source3 && source4 && source5;
        }

        public bool Execute(bool source1, bool source2, bool source3, bool source4, bool source5, bool source6)
        {
            return source1 && source2 && source3 && source4 && source5 && source6;
        }

        #endregion
    }

    [HandlerCategory(HandlerCategories.TradeMath, "LogicalOrTemplate", false)]
    [InputsCount(2, 6)]
    public sealed class Or : IBoolAccum
    {
        #region IBoolAccum Members

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2)
        {
            return Series.Or(src1, src2);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3)
        {
            return Execute(Series.Or(src1, src2), src3);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3, IList<bool> src4)
        {
            return Execute(Series.Or(src1, src2), src3, src4);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3, IList<bool> src4, IList<bool> src5)
        {
            return Execute(Series.Or(src1, src2), src3, src4, src5);
        }

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2, IList<bool> src3, IList<bool> src4, IList<bool> src5, IList<bool> src6)
        {
            return Execute(Series.Or(src1, src2), src3, src4, src5, src6);
        }

        public bool Execute(bool source1, bool source2)
        {
            return source1 || source2;
        }

        public bool Execute(bool source1, bool source2, bool source3)
        {
            return source1 || source2 || source3;
        }

        public bool Execute(bool source1, bool source2, bool source3, bool source4)
        {
            return source1 || source2 || source3 || source4;
        }

        public bool Execute(bool source1, bool source2, bool source3, bool source4, bool source5)
        {
            return source1 || source2 || source3 || source4 || source5;
        }

        public bool Execute(bool source1, bool source2, bool source3, bool source4, bool source5, bool source6)
        {
            return source1 || source2 || source3 || source4 || source5 || source6;
        }

        #endregion
    }

    //[HandlerName("Cross Over")]
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class CrossOver : ValueSupportedComparer, IDoubleCompaperHandler
    {
        #region IDoubleCompaperHandler Members

        public IList<bool> Execute(IList<double> src1, IList<double> src2)
        {
            int count = Math.Max(src1.Count, src2.Count);
            int mcount = Math.Min(src1.Count, src2.Count);
            var dst = new bool[count];
            for (int i = 1; i < mcount; i++)
            {
                dst[i] = (src1[i] > src2[i] && src1[i - 1] <= src2[i - 1]);
            }
            return dst;
        }

        #endregion

        protected override bool Execute(IList<double> src1, IList<double> src2, IList<bool> dst, int i)
        {
            if (i > 1)
            {
                dst[i] = (src1[i] > src2[i] && src1[i - 1] <= src2[i - 1]);
            }
            return dst[i];
        }
    }

    //[HandlerName("Cross Under")]
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class CrossUnder : ValueSupportedComparer, IDoubleCompaperHandler
    {
        #region IDoubleCompaperHandler Members

        public IList<bool> Execute(IList<double> src1, IList<double> src2)
        {
            int count = Math.Max(src1.Count, src2.Count);
            int mcount = Math.Min(src1.Count, src2.Count);
            var dst = new bool[count];
            for (int i = 1; i < mcount; i++)
            {
                dst[i] = (src1[i] < src2[i] && src1[i - 1] >= src2[i - 1]);
            }
            return dst;
        }

        #endregion

        protected override bool Execute(IList<double> src1, IList<double> src2, IList<bool> dst, int i)
        {
            if (i > 1)
            {
                dst[i] = (src1[i] < src2[i] && src1[i - 1] >= src2[i - 1]);
            }
            return dst[i];
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class EqualHandler : ValueSupportedComparer, IDoubleCompaperHandler
    {
        public IList<bool> Execute(IList<double> source1, IList<double> source2)
        {
            var maxCount = Math.Max(source1.Count, source2.Count);
            var minCount = Math.Min(source1.Count, source2.Count);
            var results = new bool[maxCount];

            for (var i = 0; i < minCount; i++)
                results[i] = source1[i] == source2[i];

            return results;
        }

        protected override bool Execute(IList<double> source1, IList<double> source2, IList<bool> results, int index)
        {
            return results[index] = source1[index] == source2[index];
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class GreaterHandler : ValueSupportedComparer, IDoubleCompaperHandler
    {
        public IList<bool> Execute(IList<double> source1, IList<double> source2)
        {
            var maxCount = Math.Max(source1.Count, source2.Count);
            var minCount = Math.Min(source1.Count, source2.Count);
            var results = new bool[maxCount];

            for (var i = 0; i < minCount; i++)
                results[i] = source1[i] > source2[i];

            return results;
        }

        protected override bool Execute(IList<double> source1, IList<double> source2, IList<bool> results, int index)
        {
            return results[index] = source1[index] > source2[index];
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class LessHandler : ValueSupportedComparer, IDoubleCompaperHandler
    {
        public IList<bool> Execute(IList<double> source1, IList<double> source2)
        {
            var maxCount = Math.Max(source1.Count, source2.Count);
            var minCount = Math.Min(source1.Count, source2.Count);
            var results = new bool[maxCount];

            for (var i = 0; i < minCount; i++)
                results[i] = source1[i] < source2[i];

            return results;
        }

        protected override bool Execute(IList<double> source1, IList<double> source2, IList<bool> results, int index)
        {
            return results[index] = source1[index] < source2[index];
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class GreaterOrEqualHandler : ValueSupportedComparer, IDoubleCompaperHandler
    {
        public IList<bool> Execute(IList<double> source1, IList<double> source2)
        {
            var maxCount = Math.Max(source1.Count, source2.Count);
            var minCount = Math.Min(source1.Count, source2.Count);
            var results = new bool[maxCount];

            for (var i = 0; i < minCount; i++)
                results[i] = source1[i] >= source2[i];

            return results;
        }

        protected override bool Execute(IList<double> source1, IList<double> source2, IList<bool> results, int index)
        {
            return results[index] = source1[index] >= source2[index];
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class LessOrEqualHandler : ValueSupportedComparer, IDoubleCompaperHandler
    {
        public IList<bool> Execute(IList<double> source1, IList<double> source2)
        {
            var maxCount = Math.Max(source1.Count, source2.Count);
            var minCount = Math.Min(source1.Count, source2.Count);
            var results = new bool[maxCount];

            for (var i = 0; i < minCount; i++)
                results[i] = source1[i] <= source2[i];

            return results;
        }

        protected override bool Execute(IList<double> source1, IList<double> source2, IList<bool> results, int index)
        {
            return results[index] = source1[index] <= source2[index];
        }
    }
}
