﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class BarNumber : IOneSourceHandler, IDoubleReturns, IStreamHandler, ISecurityInputs, IDoubleInputs, IIntInputs, IBooleanInputs
    {
        private sealed class IndexList : IList<double>
        {
            public IndexList(int count)
            {
                Count = count;
            }

            public IEnumerator<double> GetEnumerator()
            {
                for (var i = 0; i < Count; i++)
                    yield return i;
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                return GetEnumerator();
            }

            public void Add(double item)
            {
                throw new NotSupportedException();
            }

            public void Clear()
            {
                throw new NotSupportedException();
            }

            public bool Contains(double item)
            {
                return IndexOf(item) >= 0;
            }

            public void CopyTo(double[] array, int arrayIndex)
            {
                throw new NotSupportedException();
            }

            public bool Remove(double item)
            {
                throw new NotSupportedException();
            }

            public int Count { get; }

            public bool IsReadOnly
            {
                get { return true; }
            }

            public int IndexOf(double item)
            {
                var indexOf = (int)item;
                if (item != indexOf)
                    throw new ArgumentException(nameof(item));

                return indexOf >= 0 && indexOf < Count ? indexOf : -1;
            }

            public void Insert(int index, double item)
            {
                throw new NotSupportedException();
            }

            public void RemoveAt(int index)
            {
                throw new NotSupportedException();
            }

            public double this[int index]
            {
                get
                {
                    if (index >= 0 && index < Count)
                        return index;

                    throw new ArgumentOutOfRangeException(nameof(index));
                }
                set { throw new NotSupportedException(); }
            }
        }

        public IList<double> Execute(ISecurity seciruty)
        {
            return new IndexList(seciruty.Bars.Count);
        }

        public IList<double> Execute(IList<double> seciruty)
        {
            return new IndexList(seciruty.Count);
        }

        public IList<double> Execute(IList<int> seciruty)
        {
            return new IndexList(seciruty.Count);
        }

        public IList<double> Execute(IList<bool> seciruty)
        {
            return new IndexList(seciruty.Count);
        }
    }
}
