﻿using System.Collections.Generic;
using TSLab.DataSource;

namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class Compress : ICompressHandler
    {
        [HandlerParameter(true, "5", Min = "5", Max = "60", Step = "5")]
        public int Interval { get; set; }

        [HandlerParameter(true, "0", Min = "0", Max = "60", Step = "5")]
        public int Shift { get; set; }

        public ISecurity Execute(ISecurity source)
        {
            var interval = new Interval(Interval, source.IntervalBase);
            switch (interval.Base)
            {
                case DataIntervals.VOLUME:
                    return source.CompressToVolume(interval);
                case DataIntervals.PRICERANGE:
                    return source.CompressToPriceRange(interval);
                default:
                    return source.CompressTo(interval, Shift);
            }
        }
    }

    //[HandlerName("Compress (Advanced)")]
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class CompressAdvanced : ICompressHandler
    {
        [HandlerParameter(Default = "MINUTE", NotOptimized = true)]
        public DataIntervals IntervalBase { get; set; }

        [HandlerParameter(true, "5", Min = "5", Max = "60", Step = "5")]
        public int Interval { get; set; }

        [HandlerParameter(true, "0", Min = "0", Max = "60", Step = "5")]
        public int Shift { get; set; }

        [HandlerParameter(true, "1440", Min = "60", Max = "10080", Step = "60")]
        public int Adjustment { get; set; }

        [HandlerParameter(true, "600", Min = "0", Max = "1440", Step = "60")]
        public int AdjShift { get; set; }

        public ISecurity Execute(ISecurity source)
        {
            var interval = new Interval(Interval, IntervalBase);
            switch (interval.Base)
            {
                case DataIntervals.VOLUME:
                    return source.CompressToVolume(interval);
                case DataIntervals.PRICERANGE:
                    return source.CompressToPriceRange(interval);
                default:
                    return source.CompressTo(interval, Shift, Adjustment, AdjShift);
            }
        }
    }

    //[HandlerName("Compress to Seconds")]
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class CompressToSeconds : ICompressHandler
    {
        [HandlerParameter(true, "5", Min = "5", Max = "60", Step = "5")]
        public int Interval { get; set; }

        public ISecurity Execute(ISecurity source)
        {
            return source.CompressTo(new Interval(Interval, DataIntervals.SECONDS));
        }
    }

    //[HandlerName("Decompress")]
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class Decompres : IDecompress, IBarDoubleAccumHandler
    {
        [HandlerParameter(true, "Default", NotOptimized = true)]
        public DecompressMethodWithDef Method { get; set; }

        public IList<double> Execute(ISecurity security, IList<double> source)
        {
            return security.Decompress(source, Method);
        }
    }

    //[HandlerName("DecompressBool")]
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class DecompresBool : IDecompress, IBarBoolAccumHandler
    {
        [HandlerParameter(true, "Default", NotOptimized = true)]
        public DecompressMethodWithDef Method { get; set; }

        public IList<bool> Execute(ISecurity security, IList<bool> source)
        {
            return security.Decompress(source, Method);
        }
    }
}
