using System;
using System.Collections;
using System.Collections.Generic;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable ClassNeverInstantiated.Global

namespace TSLab.Script.Handlers
{
    //[HandlerName("Constant")]
    [HandlerCategory(HandlerCategories.TradeMath, "Const", true)]
    public sealed class ConstGen : ConstGenBase<double>, IBar2DoubleHandler, IDouble1CalculatorHandler, IDouble2DoubleHandler
    {
        [HandlerParameter]
        public double Value { get; set; }

        public IList<double> Execute(IContext context)
        {
            MakeList(context.BarsCount, Value);
            return this;
        }

        public IList<double> Execute(ISecurity source)
        {
            MakeList(source.Bars.Count, Value);
            return this;
        }

        public double Execute(double source1)
        {
            return Value;
        }

        public IList<double> Execute(IList<double> source)
        {
            MakeList(source.Count, Value);
            return this;
        }
    }

    //[HandlerName("Bool Constant")]
    [HandlerCategory(HandlerCategories.TradeMath, "BoolConst", true)]
    public class BoolConst : ConstGenBase<bool>, IBar2BoolsHandler
    {
        [HandlerParameter]
        public bool Value { get; set; }

        public IList<bool> Execute(IContext context)
        {
            MakeList(context.BarsCount, Value);
            return this;
        }

        public IList<bool> Execute(ISecurity source)
        {
            MakeList(source.Bars.Count, Value);
            return this;
        }
    }

    // useless test class

    //[HandlerName("Bool Breaker")]
    [HandlerCategory(HandlerCategories.TradeMath, "BoolConst", true)]
    public sealed class BoolBreaker : BoolConst
    {
        public override IEnumerator<bool> GetEnumerator()
        {
            return new BreakerEnumerator(m_count, m_value);
        }

        public override bool this[int index]
        {
            get { return index == m_count - 1 && m_value; }
            set { throw new InvalidOperationException(); }
        }

        private sealed class BreakerEnumerator : Enumerator
        {
            public BreakerEnumerator(int size, bool value)
                : base(size, value)
            {
            }

            public override bool Current
            {
                get { return m_cur >= m_size - 1 && m_value; }
            }
        }
    }

    public class ConstGenBase<T> : IList<T>
        where T : struct, IComparable
    {
        protected int m_count = int.MaxValue;

        protected T m_value;

        protected ConstGenBase()
        {
        }

        public ConstGenBase(int count, T value)
        {
            m_count = count;
            m_value = value;
        }

        protected void MakeList(int count, T value)
        {
            m_count = count;
            m_value = value;
        }

        #region Implementation of IEnumerable

        public virtual IEnumerator<T> GetEnumerator()
        {
            return new Enumerator(m_count, m_value);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion

        #region Implementation of ICollection<double>

        public void Add(T item)
        {
            throw new InvalidOperationException();
        }

        public void Clear()
        {
            throw new InvalidOperationException();
        }

        public bool Contains(T item)
        {
            return item.Equals(m_value);
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            for (var i = arrayIndex; i < array.Length; i++)
                array[i] = m_value;
        }

        public bool Remove(T item)
        {
            throw new InvalidOperationException();
        }

        public int Count
        {
            get { return m_count; }
        }

        public bool IsReadOnly
        {
            get { return true; }
        }

        #endregion

        #region Implementation of IList<T>

        public int IndexOf(T item)
        {
            return 0;
        }

        public void Insert(int index, T item)
        {
            throw new InvalidOperationException();
        }

        public void RemoveAt(int index)
        {
            throw new InvalidOperationException();
        }

        public virtual T this[int index]
        {
            get { return m_value; }
            set { throw new InvalidOperationException(); }
        }

        #endregion

        protected class Enumerator : IEnumerator<T>
        {
            protected readonly int m_size;
            protected readonly T m_value;
            protected int m_cur;

            public Enumerator(int size, T value)
            {
                m_size = size;
                m_cur = 0;
                m_value = value;
            }

            #region Implementation of IDisposable

            public void Dispose()
            {
            }

            #endregion

            #region Implementation of IEnumerator

            public bool MoveNext()
            {
                return ++m_cur <= m_size;
            }

            public void Reset()
            {
                m_cur = 0;
            }

            public virtual T Current
            {
                get { return m_value; }
            }

            object IEnumerator.Current
            {
                get { return Current; }
            }

            #endregion
        }
    }
}