﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class ControlledBoolConst : IOneSourceHandler, IBooleanReturns, IStreamHandler, IBooleanInputs
    {
        private static readonly IList<bool> s_emptyResult = new bool[0];

        [HandlerParameter]
        public bool Value { get; set; }

        [HandlerParameter(NotOptimized = true)]
        public bool DefaultValue { get; set; }

        public IList<bool> Execute(IList<bool> source)
        {
            if (source == null)
                throw new ArgumentNullException(nameof(source));

            if (source.Count == 0)
                return s_emptyResult;

            var firstValue = source[0];
            if (source.All(item => item == firstValue))
                return new ConstList<bool>(source.Count, firstValue);

            var result = new bool[source.Count];
            for (var i = 0; i < result.Length; i++)
                result[i] = source[i] ? Value : DefaultValue;

            return result;
        }
    }
}
