﻿using System.Collections.Generic;
using TSLab.Script.Helpers;

namespace TSLab.Script.Handlers
{
    public abstract class ExtremePos : IDouble2DoubleHandler
    {
        [HandlerParameter(true, "20", Min = "10", Max = "100", Step = "5", EditorMin = "1")]
        public int Period { get; set; }

        public IList<double> Execute(IList<double> source)
        {
            var extremeValues = GetExtremeValues(source);
            var result = new double[source.Count];

            for (var i = 1; i < result.Length; i++)
            {
                var k = 0;
                while (i - k >= 0 && extremeValues[i] != source[i - k])
                    k++;

                result[i] = k;
            }
            return result;
        }

        protected abstract IList<double> GetExtremeValues(IList<double> source);
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class HighestPos : ExtremePos
    {
        protected override IList<double> GetExtremeValues(IList<double> source)
        {
            return Series.Highest(source, Period);
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class LowestPos : ExtremePos
    {
        protected override IList<double> GetExtremeValues(IList<double> source)
        {
            return Series.Lowest(source, Period);
        }
    }
}
