﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Media;
using TSLab.DataSource;
using TSLab.Script.GraphPane;
using TSLab.Script.Optimization;

namespace TSLab.Script.Handlers
{
    [HandlerCategory(BlockCategories.GraphPaneHandler)]
    [InputsCount(1)]
    [Input(0, TemplateTypes.GRAPHPANE, Name = "TemplateLibrary.Pane")]
    public sealed class InteractiveConstGen : ConstGenBase<double>, IInteractiveConstGen
    {
        private IInteractiveSimpleLine m_interactiveSimpleLine;

        public IContext Context { get; set; }

        [HandlerParameter(Name = "Value", Default = "100", IsShown = false)]
        public OptimProperty Value { get; set; }

        [HandlerParameter(Default = "RIGHT", IsShown = false, NotOptimized = true)]
        public PaneSides PaneSide { get; set; }

        [HandlerParameter(Default = "#ff0000", IsShown = false, NotOptimized = true, Editor = "StringToColorTemplate")]
        public string Color { get; set; }

        [HandlerParameter(Default = "true", IsShown = false, NotOptimized = true)]
        public bool IsNeedRecalculate { get; set; }

        public IList<double> Execute(IGraphPane pane)
        {
            if (pane == null)
                throw new ArgumentNullException(nameof(pane));

            var id = typeof(IInteractiveConstGen).Name + "." + Value.Data.GetId();
            var container = (NotClearableContainer<InteractiveConstGen>)Context.LoadObject(id);
            container?.Content.Unsubscribe();
            InitInteractiveSimpleLine(pane);
            Subscribe();
            MakeList(Context.BarsCount, Value.Value);
            Context.StoreObject(id, new NotClearableContainer<InteractiveConstGen>(this));
            return this;
        }

        private void InitInteractiveSimpleLine(IGraphPane pane)
        {
            var id = Value.Data.GetId();
            m_interactiveSimpleLine = (IInteractiveSimpleLine)pane.GetInteractiveObject(id);
            var mediaColor = (System.Windows.Media.Color)ColorConverter.ConvertFromString(Color);
            var intColor = mediaColor.ToInt();
            MarketPoint marketPosition;

            if (m_interactiveSimpleLine != null)
            {
                marketPosition = new MarketPoint(m_interactiveSimpleLine.MarketPosition.X, Value.Value);
                if (m_interactiveSimpleLine.PaneSides == PaneSide && m_interactiveSimpleLine.Color == intColor)
                {
                    m_interactiveSimpleLine.MarketPosition = marketPosition;
                    pane.AddUnremovableInteractiveObjectId(id);
                    return;
                }
                pane.RemoveInteractiveObject(id);
            }
            else
                marketPosition = new MarketPoint(DateTime.UtcNow, Value.Value);

            m_interactiveSimpleLine = pane.AddInteractiveSimpleLine(id, PaneSide, false, intColor, InteractiveSimpleLineMode.Horizontal, marketPosition);
        }

        private void Unsubscribe()
        {
            Value.PropertyChanged -= OnValuePropertyChanged;
            m_interactiveSimpleLine.PropertyChanged -= OnInteractiveSimpleLinePropertyChanged;
        }

        private void Subscribe()
        {
            Value.PropertyChanged += OnValuePropertyChanged;
            m_interactiveSimpleLine.PropertyChanged += OnInteractiveSimpleLinePropertyChanged;
        }

        private void OnValuePropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IOptimPropertyBase.Value))
            {
                Unsubscribe();
                m_interactiveSimpleLine.MarketPosition = new MarketPoint(m_interactiveSimpleLine.MarketPosition.X, Value.Value);
                Subscribe();
            }
        }

        private void OnInteractiveSimpleLinePropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IInteractivePoint.MarketPosition))
            {
                Unsubscribe();
                ((OptimDataBase)Value.Data).Value = Value.Value = m_interactiveSimpleLine.MarketPosition.Y;
                Subscribe();
            }
            if (IsNeedRecalculate && !m_interactiveSimpleLine.IsMoving && (e.PropertyName == nameof(IInteractivePoint.MarketPosition) || e.PropertyName == nameof(IInteractivePoint.IsMoving)))
                Context.Recalc();
        }
    }
}
