﻿using System;
using System.Collections.Generic;
using TSLab.DataSource;

namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class SessionHeld : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var result = new double[bars.Count];
            var currentHeld = 0;

            for (var i = 1; i < result.Length; i++)
            {
                if (bars[i - 1].Date.Day != bars[i].Date.Day)
                    currentHeld = 0;
                else
                    currentHeld++;

                result[i] = currentHeld;
            }
            return result;
        }
    }

    public abstract class SessionBase : IBar2DoubleHandler
    {
        [HandlerParameter(true, "1", Min = "0", Max = "10", Step = "1", EditorMin = "0")]
        public int Session { get; set; }

        public IList<double> Execute(ISecurity source)
        {
            var bars = source.Bars;
            var result = new double[bars.Count];

            if (result.Length > 0)
            {
                var currentResult = new List<double>(Session + 1);
                var initialValue = GetInitialValue(bars[0]);

                for (var i = 0; i <= Session; i++)
                    currentResult.Add(initialValue);

                for (var i = 1; i < result.Length; i++)
                {
                    var bar = bars[i];
                    if (bars[i - 1].Date.Day != bar.Date.Day)
                    {
                        currentResult.RemoveAt(0);
                        currentResult.Add(GetValue(bar));
                    }
                    currentResult[Session] = GetValue(bar, currentResult);
                    result[i] = currentResult[0];
                }
            }
            return result;
        }

        protected abstract double GetInitialValue(IDataBar bar);

        protected abstract double GetValue(IDataBar bar);

        protected abstract double GetValue(IDataBar bar, IReadOnlyList<double> currentResult);
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class SessionOpen : SessionBase
    {
        protected override double GetInitialValue(IDataBar bar)
        {
            return bar.Open;
        }

        protected override double GetValue(IDataBar bar)
        {
            return bar.Open;
        }

        protected override double GetValue(IDataBar bar, IReadOnlyList<double> currentResult)
        {
            return currentResult[Session];
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class SessionClose : SessionBase
    {
        protected override double GetInitialValue(IDataBar bar)
        {
            return bar.Open;
        }

        protected override double GetValue(IDataBar bar)
        {
            return bar.Close;
        }

        protected override double GetValue(IDataBar bar, IReadOnlyList<double> currentResult)
        {
            return bar.Close;
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class SessionHigh : SessionBase
    {
        protected override double GetInitialValue(IDataBar bar)
        {
            return bar.High;
        }

        protected override double GetValue(IDataBar bar)
        {
            return bar.High;
        }

        protected override double GetValue(IDataBar bar, IReadOnlyList<double> currentResult)
        {
            return Math.Max(bar.High, currentResult[Session]);
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class SessionLow : SessionBase
    {
        protected override double GetInitialValue(IDataBar bar)
        {
            return bar.Low;
        }

        protected override double GetValue(IDataBar bar)
        {
            return bar.Low;
        }

        protected override double GetValue(IDataBar bar, IReadOnlyList<double> currentResult)
        {
            return Math.Min(bar.Low, currentResult[Session]);
        }
    }
}
