﻿using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using TSLab.Script.Helpers;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable ClassNeverInstantiated.Global
// ReSharper disable InconsistentNaming
// ReSharper disable CompareOfFloatsByEqualityOperator
namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.Indicators)]
    public sealed class StochK : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            var high = Context.GetData("Highest", new[] { Period.ToString(CultureInfo.InvariantCulture), source.CacheName },
                                       () => Series.Highest(source.HighPrices, Period));
            var low = Context.GetData("Lowest", new[] { Period.ToString(CultureInfo.InvariantCulture), source.CacheName },
                                       () => Series.Lowest(source.LowPrices, Period));
            var closes = source.ClosePrices;
            var list = new double[closes.Count];
            for (int i = 0; i < closes.Count; i++)
            {
                var hl = high[i] - low[i];
                var stochK = hl == 0 ? 0 : 100 * (closes[i] - low[i]) / hl;
                list[i] = stochK;
            }
            return list;
        }

        public IContext Context { get; set; }
    }

    [HandlerCategory(HandlerCategories.Indicators)]
    //[HandlerName("StochRSI")]
    [SuppressMessage("StyleCopPlus.StyleCopPlusRules", "SP0100:AdvancedNamingRules", Justification = "Reviewed. Suppression is OK here.")]
    public sealed class StochRSI : BasePeriodIndicatorHandler, IBar2DoubleHandler, IContextUses
    {
        public IList<double> Execute(ISecurity source)
        {
            var rsi = Context.GetData("RSI", new[] { Period.ToString(CultureInfo.InvariantCulture), source.CacheName },
                                       () => Series.RSI(source.ClosePrices, Period));
            var high = Series.Highest(rsi, Period);
            var low = Series.Lowest(rsi, Period);
            var list = new double[rsi.Count];
            for (int i = 0; i < rsi.Count; i++)
            {
                var hl = high[i] - low[i];
                var stochRSI = hl == 0 ? 0 : 100 * (rsi[i] - low[i]) / hl;
                list[i] = stochRSI;
            }
            return list;
        }

        public IContext Context { get; set; }
    }
}
