﻿using System;
using System.Collections.Generic;
using TSLab.Script.Helpers;

// ReSharper disable UnusedMember.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable ClassNeverInstantiated.Global
// ReSharper disable InconsistentNaming
namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.TradeMath)]
    public class Flip : IBoolAccum
    {
        public IList<bool> Execute(IList<bool> source1, IList<bool> source2)
        {
            return Calc(source1, source2);
        }

        public bool m_last;

        public bool Execute(bool source1, bool source2)
        {
            m_last = Calc(source1, source2, m_last);
            return m_last;
        }

        public static IList<bool> Calc(IList<bool> source1, IList<bool> source2)
        {
            var res = new bool[source1.Count];
            for (int i = 1; i < source1.Count; i++)
            {
                var last = i > 0 && res[i - 1];
                res[i] = Calc(source1[i], source2[i], last);
            }
            return res;
        }

        public static bool Calc(bool source1, bool source2, bool last)
        {
            var res = last;
// ReSharper disable ConvertIfToOrExpression
            if (!res & source1)
// ReSharper restore ConvertIfToOrExpression
            {
                res = true;
            }
            if (res & source2)
            {
                res = false;
            }
            return res;
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public class Hold : BasePeriodIndicatorHandler, IBoolConvertor
    {
        public IList<bool> Execute(IList<bool> source)
        {
            return Calc(source, Period);
        }

        private int m_savedNum = -1;

        public bool Execute(bool source, int num)
        {
            return Calc(source, Period, num, ref m_savedNum);
        }

        public static IList<bool> Calc(IList<bool> source, int period)
        {
            var res = new bool[source.Count];
            int savedNum = -1;
            for (int i = 1; i < source.Count; i++)
            {
                res[i] = Calc(source[i], period, i, ref savedNum);
            }
            return res;
        }

        public static bool Calc(bool source, int period, int num, ref int savedNum)
        {
            if (source)
            {
                savedNum = num;
            }
            if (savedNum >= 0 && (num - savedNum) <= period)
            {
                source = true;
            }
            return source;
        }
    }

    [HandlerCategory(HandlerCategories.TradeMath)]
    public sealed class Shift : DoubleStreamAndValuesHandlerWithPeriod
    {
        private double m_firstValue;
        private Queue<double> m_queue;

        public override bool IsGapTolerant
        {
            get { return true; }
        }

        public override IList<double> Execute(IList<double> source)
        {
            var result = Series.Shift(source, Period);
            return result;
        }

        protected override void InitExecuteContext()
        {
            if (IsSimple)
                m_firstValue = m_executeContext.Source;
            else
            {
                m_queue = new Queue<double>(Period);
                for (var i = 0; i < Period; i++)
                    m_queue.Enqueue(m_executeContext.Source);
            }
        }

        protected override void ClearExecuteContext()
        {
            m_firstValue = 0;
            m_queue = null;
        }

        protected override void InitForGap()
        {
            if (IsSimple)
                return;

            var firstIndex = Math.Max(m_executeContext.LastIndex + 1, m_executeContext.Index - Period);
            for (var i = firstIndex; i < m_executeContext.Index; i++)
            {
                var result = m_queue.Dequeue();
                var source = m_executeContext.GetSourceForGap(i);
                m_queue.Enqueue(source);
            }
        }

        protected override double Execute()
        {
            if (IsSimple)
                return m_firstValue;

            var result = m_queue.Dequeue();
            m_queue.Enqueue(m_executeContext.Source);
            return result;
        }

        protected override bool IsSimple
        {
            get { return Context.BarsCount <= Period; }
        }
    }
}
