﻿#if DEBUG
using System;
using System.Collections.Generic;
using System.Linq;

namespace TSLab.Script.Handlers
{
    [HandlerCategory(HandlerCategories.MarketPosition)]
    public sealed class TradeStatisticsExtremumValueHandler : ITradeStatisticsExtremumValueHandler
    {
        public IList<double> Execute(IBaseTradeStatisticsWithKind tradeStatistics)
        {
            var histograms = tradeStatistics.GetHistograms();
            var barsCount = tradeStatistics.Bars.Count;

            if (histograms.Count == 0)
                return new ConstGenBase<double>(barsCount, double.NaN);

            var extremumValues = new double[barsCount];
            var orderedHistograms = histograms.OrderBy(item => item.FirstBarIndex).ToArray();
            var iMax = orderedHistograms[0].FirstBarIndex - 1;
            var lastExtremumValue = double.NaN;

            for (var i = 0; i < iMax; i++)
                extremumValues[i] = lastExtremumValue;

            foreach (var histogram in orderedHistograms)
            {
                lastExtremumValue = GetExtremumValue(tradeStatistics, histogram, lastExtremumValue);
                for (var i = histogram.FirstBarIndex; i <= histogram.LastBarIndex; i++)
                    extremumValues[i] = lastExtremumValue;
            }
            return extremumValues;
        }

        private static double GetExtremumValue(IBaseTradeStatisticsWithKind tradeStatistics, ITradeHistogram histogram, double lastExtremumValue)
        {
            var bars = histogram.Bars;
            if (bars.Count == 0)
                return lastExtremumValue;

            if (bars.Count == 1)
                return bars[0].GetPrice();

            TradeHistogramBar maxBar, minBar;
            maxBar = minBar = bars[0];

            double maxValue, minValue;
            maxValue = minValue = tradeStatistics.GetValue(minBar);

            foreach (var bar in bars.Skip(1))
            {
                var value = tradeStatistics.GetValue(bar);
                if (value > maxValue)
                {
                    maxBar = bar;
                    maxValue = value;
                }
                else if (value < minValue)
                {
                    minBar = bar;
                    minValue = value;
                }
            }
            return Math.Abs(maxValue) >= Math.Abs(minValue) ? maxBar.GetPrice() : minBar.GetPrice();
        }
    }
}
#endif
