﻿#if DEBUG
using System;
using TSLab.DataSource;
using TSLab.Utils;

namespace TSLab.Script.Handlers
{
    public sealed class TradeStatisticsHandler : BaseTradeStatisticsHandler<ITradeStatisticsWithKind>, ITradeStatisticsHandler
    {
        [HandlerParameter(true, "1", Min = "1", Max = "365", Step = "1")]
        public int TimeFrame { get; set; }

        [HandlerParameter(true, nameof(TimeFrameUnit.Hour), NotOptimized = true)]
        public TimeFrameUnit TimeFrameUnit { get; set; }

        [HandlerParameter(true, "2000000", Min = "1", Max = "2000000", Step = "1")]
        public int TopTimeFrame { get; set; }

        [HandlerParameter(true, nameof(TimeFrameUnit.Day), NotOptimized = true)]
        public TimeFrameUnit TopTimeFrameUnit { get; set; }

        [HandlerParameter(true, nameof(TimeFrameKind.FromNowToPast), NotOptimized = true)]
        public TimeFrameKind TimeFrameKind { get; set; }

        public override ITradeStatisticsWithKind Execute(ISecurity security)
        {
            var timeFrame = TimeFrameFactory.Create(TimeFrame, TimeFrameUnit);
            var topTimeFrame = TimeFrameFactory.Create(TopTimeFrame, TopTimeFrameUnit);

            if (topTimeFrame.Ticks % timeFrame.Ticks != 0)
                throw new InvalidOperationException(string.Format(RM.GetString("TopTimeFrameMustBeDivisableByTimeFrame"), ToString(TopTimeFrame, TopTimeFrameUnit), ToString(TimeFrame, TimeFrameUnit)));

            var id = string.Join(".", security.Symbol, security.Interval, CombinePricesCount, TimeFrame, TimeFrameUnit, TimeFrame, TimeFrameUnit, TimeFrameKind);
            var tradeStatistics = Context.GetTradeStatistics(id, () => new TradeStatistics(security, CombinePricesCount, timeFrame, TimeFrameUnit, topTimeFrame, TimeFrameKind));
            return new TradeStatisticsWithKind(tradeStatistics, Kind);
        }

        private static string ToString(int timeFrame, TimeFrameUnit timeFrameUnit)
        {
            return timeFrame + ":" + timeFrameUnit.GetDescription();
        }
    }
}
#endif
